;FOR MORE INFORMATION:
; go to http://www.delorie.com/djgpp/doc/rbinter
; for Ralph Brown's interrupt list


%macro m_FastDisplayString 0
	;this macro displays the '$' terminated string
	;at DS:SI to the screen through the BIOS. this
	;is extremely faster on the calc than using
	;the comparable DOS call.

	;For details on the recommended use of this
	;macro, see macros.txt.

	;   * MAY NOT WORK IN PC GRAPHICS MODE *

	mov ah,0x0E
	xor bx,bx
	mov cx,0x0001
%%DLp:	mov al,[ds:si]
	cmp al,'$'
	je %%Done
	int 0x10
	inc si
	jmp short %%DLp
%%Done:
%endmacro

%macro m_DisplayNumberAX 0
	;this macro displays the number in AX to the
	;screen (through the DOS display character
	;call) in decimal format.

	;For details on the recommended use of this
	;macro, see macros.txt.

	mov bx,0x000a
	xor cx,cx
%%LblA:	inc cx
	xor dx,dx
	div bx
	push dx
	or ax,ax
	jnz %%LblA
%%LblB:	pop dx
	add dl,0x30
	mov ah,0x02
	int 0x21
	loop %%LblB
%endmacro

%macro m_DisplayNumberAXh 0
	;similar to the previous macro, except the
	;number is displayed in 4 digit hexadecimal.

	;For details on the recommended use of this
	;macro, see macros.txt.

	mov bx,ax
	mov ah,0x02
	mov dl,'0'
	cmp bx,0x0FFF
	ja %%Z0
	cmp bx,0x00FF
	ja %%Z1
	cmp bx,0x000F
	ja %%Z2
	int 0x21
%%Z2:	int 0x21
%%Z1:	int 0x21
%%Z0:	mov ax,bx
	mov bx,0x0010
	xor cx,cx
%%LblA:	inc cx
	xor dx,dx
	div bx
	push dx
	or ax,ax
	jnz %%LblA
%%LblB:	pop dx
	add dl,0x30
	cmp dl,57
	jbe %%LblC
	add dl,0x07
%%LblC:	mov ah,0x02
	int 0x21
	loop %%LblB
%endmacro

%macro m_DisplayNumberALh 0
	;similar to the previous macro, except
	;only the value in AL is used, and is
	;displayed in 2 digit hexadecimal.

	;For details on the recommended use of this
	;macro, see macros.txt.

	xor ah,ah
	mov bx,ax
	cmp bx,0x000F
	ja %%Z0
	mov ah,0x02
	mov dl,'0'
	int 0x21
%%Z0:	mov ax,bx
	mov bx,0x0010
	xor cx,cx
%%LblA:	inc cx
	xor dx,dx
	div bx
	push dx
	or ax,ax
	jnz %%LblA
%%LblB:	pop dx
	add dl,0x30
	cmp dl,57
	jbe %%LblC
	add dl,0x07
%%LblC:	mov ah,0x02
	int 0x21
	loop %%LblB
%endmacro

%macro m_GetNumberAXh 0
	;this useful macro allows the user to enter a 4
	;digit hexadecimal number, which is stored in AX.

	;left arrow is allowed to backspace, and if CL=1,
	;allows the user to cancel number entry if there
	;is nothing to backspace over.

	;the entry takes place at the current cursor
	;position, and entry is completed when the user
	;enters the 4th digit.

	;CL value allows cancel - 0=not allowed, else allowed
	;CH MUST be cleared prior to running, unless
	;editing a partial value

	;Return values:
	;AX=Number entered
	;CL=0 if operation is canceled.
	;CL is _NOT_ explicitly set to 1 if entry took place.
	;(If CL=0 before the operation, it will = 0 after)

	;Registers destroyed: BX,CH,DX

	;For details on the recommended use of this
	;macro, see macros.txt.

	mov ah,0x02
	mov dl,'?'
	int 0x21
	mov dl,0x08
	int 0x21
%%GK:	mov ah,0x08
	int 0x21
	cmp al,0
	je %%PN
	cmp al,48
	jl %%GK ;no useful keypress - too low
	cmp al,57
	jg %%PGD
	;display AL because we know it's valid
	xchg al,dl ;in preparation to display
	mov ah,0x02
	int 0x21 ;display
	sub dl,48 ;get DL with the actual value
%%MRG:	inc ch ;increment the character count
	times 4 shl bx,1 ;shift BX in preparation for the value
	or bl,dl ;merge in the nibble
	cmp ch,4
	je %%Done
	mov dl,'?'
	int 0x21
	mov dl,0x08 ;backspace
	int 0x21
	jmp short %%GK ;next key
%%PGD:	;AL has value over decimal range
	cmp al,65
	jb %%GK ;more useless input
	cmp al,70
	ja %%GK ;even more dumb, useless keypresses
	;ok now we have a valid hex number
	xchg al,dl ;in preparation to display
	mov ah,0x02
	int 0x21 ;display
	sub dl,55 ;change the ascii character to the real number
	jmp short %%MRG ;we just skip back to the previous merge code
%%PN:	;process a null byte
	int 0x21 ;get the second byte (AH still has 'read keyboard')
	cmp al,75 ;backspace (left arrow technically)
	jne %%skBK ;skip backspace code (ends up being useless value)
	cmp ch,00
	ja %%doBK ;there have been characters entered, so delete one
	;user is trying to cancel
	cmp cl,00
	je %%GK ;not allowed
	mov cl,00 ;set cancel flag
	jmp short %%Done ;exit
%%doBK:	times 4 shr bx,1 ;shift BX right 4 bits
	dec ch ;decrement the character counter
	mov ah,0x02
	mov dl,' ' ;delete the '?' cursor
	int 0x21
	call %%BKSP
	int 0x21 ;backspace again
	mov dl,'?'
	int 0x21
	call %%BKSP
	jmp short %%GK ;loop back to the get keypress part
%%skBK: ;nothing special to process if it's not a backspace here
%%UNK:	;process the unknown value - just ignore it :)
	jmp short %%GK ;loop back to the get keypress part
%%BKSP: mov ah,0x02 ;a mini function that displays a backspace
	mov dl,0x08
	int 0x21
	ret
%%Done:	mov AX,BX ;put the result in AX
%endmacro

%macro m_GetNumberALh 0
	;very similar to the previous macro, except the result
	;is returned in AL, and only 2 digits are entered.
	;to see how this works, look at the extra comments in the
	;previous one.

	;For details on the recommended use of this
	;macro, see macros.txt.

	mov ah,0x02
	mov dl,'?'
	int 0x21
	mov dl,0x08
	int 0x21
%%GK:	mov ah,0x08
	int 0x21
	cmp al,0
	je %%PN
	cmp al,48
	jl %%GK ;nothing useful
	cmp al,57
	jg %%PGD
	;display AL because we know it's valid
	xchg al,dl ;in preparation to display
	mov ah,0x02
	int 0x21 ;display
	sub dl,48 ;get DL with the actual value
%%MRG:	inc ch ;increment the character count
	times 4 shl bx,1 ;shift BX in preparation for the value
	or bl,dl ;merge in the nibble
	cmp ch,2
	je %%Done
	mov dl,'?'
	int 0x21
	mov dl,0x08 ;backspace
	int 0x21
	jmp short %%GK ;next key
%%PGD:	;AL has value over decimal range
	cmp al,65
	jb %%GK ;more useless input
	cmp al,70
	ja %%GK ;even more crap
	;ok now we have a valid hex number
	xchg al,dl ;in preparation to display
	mov ah,0x02
	int 0x21 ;display
	sub dl,55
	;we just skip back to the previous merge code
	jmp short %%MRG
%%PN:	;process a null byte
	int 0x21 ;get the second byte
	cmp al,75 ;backspace (left arrow technically)
	jne %%skBK
	cmp ch,00
	ja %%doBK
	;user is trying to cancel
	cmp cl,00
	je %%GK ;not allowed
	mov cl,00
	jmp short %%Done
%%doBK:	times 4 shr bx,1
	dec ch
	mov ah,0x02
	mov dl,' '
	int 0x21
	call %%BKSP
	int 0x21 ;backspace again
	mov dl,'?'
	int 0x21
	call %%BKSP
	jmp short %%GK
%%skBK:
%%UNK:	jmp short %%GK ;eh... whatever that was they pressed...
%%BKSP: mov ah,0x02
	mov dl,0x08
	int 0x21
	ret
%%Done:	mov ax,bx ;AL contains value, everything else is undefined
%endmacro

%macro m_DisplayMBMP2 0
	;Displays a 2 color MiniBMP
	;
	;on entry: DS:SI points to a MiniBMP data block
	;          CH is the number of columns to skip drawing
	;          CL is the left column position (0 = first)
	;          DH is the number of rows to skip drawing
	;          DL is the upper row position (0 = first)
	;          BH is the number of columns to draw (0 = all)
	;          BL is the number of rows to draw (0 = all)
	;   	
	;return values:  None
	;registers destroyed: AX, BL (set to 0), CX, DX, SI, flags

	cld
	push di		;preserve DI
	push bp		;preserve BP
	inc si 		;skip the type indicator
	
	;first we need to fix up the number of columns to display
	mov al,[si]	;get the number of columns
	sub al,ch	;subtract the number skipped
	or bh,bh	;check if BH is zero
	jz %%AdBH	;we need to adjust the columns for sure here
	cmp al,bh
	jae %%NABH	;no need to adjust BH
%%AdBH:	mov bh,al	;set BH to draw remaining columns
%%NABH: 		
	
	;calculate how many bytes to skip ahead to get at the first row

	xor ah,ah	;clear AH
	lodsb		;get the number of columns again
	push dx		;preserve DX
	mov dl,0x08
	div dl		;how many bytes does it take to store?
	or ah,ah
	jz %%NoRU	;we don't need to round up if there's no remainder
	inc al		;round up
%%NoRU: pop dx
	mul dh		;find how many bytes to skip
	push ax		;we'll need that result in a sec...

	;now we need to adjust the number of rows to display

	lodsb		;now get the number of rows
	sub al,dh	;subtract the number skipped
	or bl,bl	;check if BL is zero
	jz %%AdBL	;we need to adjust the columns for sure here
	cmp al,bl
	jae %%NABL	;no need to adjust BL
%%AdBL:	mov bl,al	;set BL to draw remaining columns
%%NABL: 
	
	pop ax		;get the number of bytes to skip back
	add si,ax	;we're now at the first row we need to display

	;calculate how many bytes to skip before each row
	xor ah,ah
	mov al,ch
	mov dh,0x08
	div dh 		;AL now has the number of full bytes to skip
			;AH has the number of bits to skip
	push ax
	xor ah,ah
	mov di,ax	;number of bytes to skip each row
	pop ax
	shr ax,0x08
	mov bp,ax	;number of bits to skip

	xor ch,ch	;clear CH
	xor dh,dh	;clear DH

%%RLp:	;this is the loop that displays a row
	push bx 	;preserve BH

	push cx		;save the left column position

	;maunually set up the first byte of data:
	add si,di
	mov ah,0x08
	lodsb		;get the data byte
	push cx
	mov cx,bp
	shl al,cl
	sub ah,cl
	pop cx		;now set up to enter the column loop
	jmp short %%CLp	;we set up the first byte already

%%CLpB:	;get a byte from the data to display
	or ah,0x08	;we have 8 bits
	lodsb		;load a data byte
%%CLp:	;this is the loop that displays a column (pixel)
	push ax
	push bx
	test al,0x80
	mov al,0x00
	jz %%PxlZ
	or al,0x01
%%PxlZ:	xor bx,bx
	mov ah,0x0c
	int 0x10
	cld
	pop bx
	pop ax
	
	shl al,1
	dec ah
	inc cx		;next display column
	dec bh		;one less to display
	jz %%CDun	;display next column if we're not done
	or ah,ah
	jz %%CLpB
	jmp short %%CLp
%%CDun:	pop cx		;restore the left column position
	pop bx 		;restore BH
	inc dx
	dec bl 		;one row down...
	jnz %%RLp

	pop bp		;restore BP
	pop di		;restore DI
	;we're done
%endmacro